import os
import sys
import pickle
import platform

from .Bot import Bot

if "android" in platform.platform():
    LOGO = """
▗▄▄▄▖▗▄▄▄▖▗▖   ▗▄▄▄▖▗▖  ▗▖ ▗▄▖ ▗▖   
  █  ▐▌   ▐▌   ▐▌   ▐▛▚▞▜▌▐▌ ▐▌▐▌   
  █  ▐▛▀▀▘▐▌   ▐▛▀▀▘▐▌  ▐▌▐▛▀▜▌▐▌   
  █  ▐▙▄▄▖▐▙▄▄▖▐▙▄▄▖▐▌  ▐▌▐▌ ▐▌▐▙▄▄▖ V1.2.20
  Telegram Bot Control Toolkit  
                            
"""
else:
    LOGO = """
████████╗███████╗██╗     ███████╗███╗   ███╗ █████╗ ██╗     
╚══██╔══╝██╔════╝██║     ██╔════╝████╗ ████║██╔══██╗██║     
   ██║   █████╗  ██║     █████╗  ██╔████╔██║███████║██║     
   ██║   ██╔══╝  ██║     ██╔══╝  ██║╚██╔╝██║██╔══██║██║     
   ██║   ███████╗███████╗███████╗██║ ╚═╝ ██║██║  ██║███████╗
   ╚═╝   ╚══════╝╚══════╝╚══════╝╚═╝     ╚═╝╚═╝  ╚═╝╚══════╝ V1.2.20
    Telegram Bot Control Toolkit
    
"""


def write_cache(bot: Bot):

    id = bot.token.split(":")[0]

    if not os.path.exists(f"{id}_cache"):
        os.makedirs(f"{id}_cache")

    with open(f"{id}_cache/bot.pkl", "wb") as file:
        pickle.dump(bot, file)

    with open(f"{id}_cache/channels.pkl", "wb") as file:
        pickle.dump(bot.channels, file)


def read_cache(token: str):

    id = token.split(":")[0]

    if os.path.exists(f"{id}_cache"):
        with open(f"{id}_cache/bot.pkl", "rb") as file:
            bot = pickle.load(file)

        with open(f"{id}_cache/channels.pkl", "rb") as file:
            bot.channels = pickle.load(file)

        # Check if bot still exists
        bot.get_me()

        return bot


def clear_screen():
    if os.name == "nt":
        os.system("cls")
    else:
        os.system("clear")


def remove_last_lines(n=1):
    for _ in range(n):
        sys.stdout.write("\033[F")  # Cursor up one line
        sys.stdout.write("\033[K")  # Clear to the end of the line


def chat_history(bot: Bot, chat_id: str):
    messages = bot.channels[chat_id].get_messages()
    for message in messages:
        print(message)


def main_menu(token: str | None = None, bot: Bot | None = None):
    clear_screen()

    print(LOGO)

    if not token:
        token = input("[+] Enter bot token > ")
        remove_last_lines()

        bot = read_cache(token)

    if not bot:
        bot = Bot(token)

    print("[Main Menu]\n")

    print(f"[+] Bot {bot.first_name} ({bot.username}) loaded successfully.")

    bot.update()

    if bot.chat_count > 1:
        print(f"\033[1m[!] Bot is part of {bot.chat_count} channels\033[0m\n")

        for i, chat in enumerate(bot.chat_list):
            print(f"{i+1}. Go to channel : {chat.split('$$$$$')[0]}")

        print(f"{bot.chat_count + 1}. Enter a channel ID.")
        print(f"{bot.chat_count + 2}. Get channels updates.")
        print("0. Exit and write cache.\n")

        case = input(">>> ")

        if case == "0":
            write_cache(bot)
            sys.exit(0)
        elif case.isdigit() and int(case) <= bot.chat_count:
            chat_id = bot.chat_list[int(case) - 1].split("$$$$$")[1]
            chat_name = bot.chat_list[int(case) - 1].split("$$$$$")[0]
            channel_menu(bot, chat_id, chat_name)
        elif case == str(bot.chat_count + 1):
            chat_id = input("[+] Enter channel ID > ")
            if not bot.is_in_channel(chat_id):
                print("[-] Error: Bot is not in this channel.")
            else:
                channel_menu(bot, chat_id)
        elif case == str(bot.chat_count + 2):
            if bot.update():
                print("[+] Channels updated successfully.")
            else:
                print("[-] No updates found.")
        else:
            print("[!] Invalid option.")
    elif bot.chat_count == 1:
        print(f"\n1. Go to channel: {bot.chat_list[0].split('$$$$$')[0]}")
        print("2. Enter a channel ID.")
        print("3. Get channels updates.")
        print("0. Exit and write cache.")

        case = input("\n>>> ")

        if case == "1":
            channel_menu(bot)
        elif case == "2":
            chat_id = input("[+] Enter channel ID > ")
            if not bot.is_in_channel(chat_id):
                print("[-] Error: Bot is not in this channel.")
            else:
                channel_menu(bot, chat_id)
        elif case == "3":
            if bot.update():
                print("[+] Channels updated successfully.")
            else:
                print("[-] No updates found.")
        elif case == "0":
            write_cache(bot)
            sys.exit(0)

    else:
        print("[-] Couldn't find any channel.\n")

        print("1. Enter a channel ID.")
        print("2. Get channels updates.")
        print("0. Exit and write cache.")

        case = input("\n>>> ")

        if case == "0":
            write_cache(bot)
            sys.exit(0)
        elif case == "1":
            chat_id = input("[+] Enter channel ID > ")
            if not bot.is_in_channel(chat_id):
                print("[-] Error: Bot is not in this channel.")
            else:
                channel_menu(bot, chat_id)
        elif case == "2":
            if bot.update():
                print("[+] Channels updated successfully.")
            else:
                print("[-] No updates found.")
        else:
            print("[!] Invalid option")

    input("\n[+] Press any key to go back...")

    main_menu(token, bot)


def print_channel_informations(bot: Bot, chat_id: str):

    invite_link = bot.channels[chat_id].invite_link
    permissions = bot.channels[chat_id].bot_permissions
    admins = bot.channels[chat_id].admins

    for index, permission in enumerate(permissions):
        if permission in ["can_send_messages", "can_delete_messages"]:
            permissions[index] = f"\033[1m{permission}\033[0m"

    print(f"[+] Channel ID: {chat_id}")
    print(f"[+] Invite Link: {invite_link}")

    print(f"[+] Permissions:", end=" ")
    print(*permissions, sep=", ")

    print(f"\n[+] Users: {bot.channels[chat_id].user_count}")

    print(f"[+] Administrators: {len(admins)}")
    for admin in admins:
        print(f"    - {admin}")


def channel_menu(bot: Bot, chat_id: str | None = None, chat_name: str | None = None):
    clear_screen()

    if not chat_id:
        chat_id = bot.chat_list[0].split("$$$$$")[1]

    bot.add_channel(chat_id)

    if not chat_name:
        chat_name = bot.channels[chat_id].name

    print(LOGO)

    if not bot.is_in_channel(chat_id):
        print(f"[Channel: {chat_name}] - Bot is not in the channel anymore.\n")

        print(f"[+] Channel ID: {chat_id}\n")

        print("1. List all messages.")
        print("2. Download files.")
        print("3. Export all text messages.")
        print("0. Go back.")

        case = input("\n>>> ")
        print()

        if case == "1":
            print("Not implemented yet.")

        elif case == "2":
            print("Not implemented yet.")

        elif case == "3":
            print("Not implemented yet.")

        elif case == "0":
            main_menu(bot.token, bot)

        else:
            print("[!] Invalid option.")

    else:
        print(f"[Channel: {chat_name}]\n")

        print_channel_informations(bot, chat_id)

        print("")

        print("1. List all messages.")
        print("2. Send a message.")
        print("3. Send a file.")
        print("4. Download files.")
        print("5. Delete all messages.")
        print("6. Export all text messages.")
        print("7. Leave channel.")
        print("0. Go back.")

        case = input("\n>>> ")

        if case == "1":
            chat_history(bot, chat_id)

        elif case == "2":
            message = input("[+] Enter message > ")

            if bot.send_message(chat_id, message):
                print("[+] Message sent successfully.")
            else:
                print("[-] Error: Message could not be sent.")

        elif case == "3":
            try:
                file_path = input("Enter the path of the file to send > ")
                if bot.send_file(chat_id, file_path):
                    print("[+] File sent successfully.")
                else:
                    print("[-] Error: File could not be sent.")
            except FileNotFoundError:
                print("[-] Error: File not found.")

        elif case == "4":
            file_menu(bot, chat_id, chat_name)

        elif case == "5":
            print(
                "[?] This will only delete messages sent less than 48 hours ago, because of Telegram's limitations."
            )

            messages_count = bot.delete_all_messages(chat_id)
            remove_last_lines()

            if messages_count == 0:
                print("[-] No messages deleted.")
            else:
                print(f"[+] {messages_count} messages deleted.")

        elif case == "6":
            bot.export_text_messages(chat_id)
            print(
                f"[+] Messages exported to ./{chat_id.replace('-', '')}/messages.txt."
            )

        elif case == "7":
            if bot.leave_channel(chat_id):
                print("[+] Successfully left the chat.")
            else:
                print("[-] Couldn't leave the chat.")

        elif case == "0":
            main_menu(bot.token, bot)

        else:
            print("[!] Invalid option.")

    input("\n[+] Press any key to go back...")

    channel_menu(bot, chat_id, chat_name)


def file_menu(bot: Bot, chat_id: str, chat_name: str):
    clear_screen()

    print(LOGO)

    print(f"[File Download Menu] - Channel: {chat_name}\n")

    file_count_dict, document_extensions = bot.get_file_count(chat_id)

    file_count = sum(file_count_dict.values())

    if file_count == 0:
        print("[-] No files found.\n")
        input("[+] Press any key to go back...")
        return channel_menu(bot, chat_id, chat_name)

    print(f"[+] Files Count: {file_count}")

    print("[+] File Types:")
    for file_type, count in file_count_dict.items():
        if count > 0:
            print(f"    - {file_type.capitalize()}: {count}")

    if document_extensions:
        print("[+] Document Extensions:")
        for extension, count in document_extensions.items():
            print(f"    - {extension}: {count}")

    print("")
    print("1. Download all files.")

    if document_extensions:
        print("2. Download all documents.")
        if file_count_dict["photo"] > 0 or file_count_dict["video"] > 0:
            print("3. Download all media.")
            print("0. Go back.")

            case = input("\n>>> ")

            if case == "1":
                bot.download_all_files(chat_id)
            elif case == "2":
                bot.download_all_files(chat_id, "documents")
            elif case == "3":
                bot.download_all_files(chat_id, "media")
            elif case == "0":
                channel_menu(bot, chat_id, chat_name)
            else:
                print("[!] Invalid option.")
        else:
            print("0. Go back.")

            case = input("\n>>> ")

            if case == "1":
                bot.download_all_files(chat_id)
            elif case == "2":
                bot.download_all_files(chat_id, "documents")
            elif case == "0":
                channel_menu(bot, chat_id, chat_name)
            else:
                print("[!] Invalid option.")

    elif file_count_dict["photo"] > 0 or file_count_dict["video"] > 0:
        print("2. Download all media.")
        print("0. Go back.")

        case = input("\n>>> ")

        if case == "1":
            bot.download_all_files(chat_id)
        elif case == "2":
            bot.download_all_files(chat_id, "media")
        elif case == "0":
            channel_menu(bot, chat_id, chat_name)
        else:
            print("[!] Invalid option.")

    input("[+] Press any key to go back...")
    file_menu(bot, chat_id, chat_name)


def main():
    try:
        main_menu()
    except KeyboardInterrupt or EOFError:
        print("\n[-] Exiting...")
        sys.exit(0)



class User:
    def __init__(
        self,
        id: int,
        is_bot: bool,
        status: str,
        first_name: str,
        username: str | None,
        is_anonymous: bool,
    ):
        self.id = id
        self.is_bot = is_bot
        self.status = status
        self.first_name = first_name
        self.username = username
        self.is_anonymous = is_anonymous

    def __str__(self):
        if self.username:
            return f"id: {self.id}, first_name: {self.first_name}, username: {self.username}, status: {self.status}, is_bot: {self.is_bot}, is_anonymous: {self.is_anonymous}"
        else:
            return f"id: {self.id}, first_name: {self.first_name}, status: {self.status}, is_bot: {self.is_bot}, is_anonymous: {self.is_anonymous}"



import os
import sys
import requests

from .Channel import Channel


class Bot:
    """
    Bot class to interact with Telegram Bot API.

    Attributes:
        token (str): Bot token.
        first_name (str): Bot's first name.
        username (str): Bot's username.
        json_updates (dict): JSON response from getUpdates.
        chat_count (int): Number of chats bot is part of.
        chat_list (list): List of chats bot is part of.
        commands (list): List of commands.
        channels (dict): Dictionary of channels bot is part of.

    Methods:
        get_me: Get bot information.
        get_updates: Get updates from bot.
        update: Update bot's JSON updates.
        get_chats: Get list of chats bot is part of.
        add_channel: Add channel to bot's channels.
        leave_channel: Leave a channel.
        is_in_channel: Check if bot is part of a channel.
        get_file_count: Get file count in a channel.
        download_file: Download a file from a channel.
        download_all_files: Download all files from a channel.
        send_message: Send a message to a channel.
        delete_message: Delete a message from a channel.
        delete_all_messages: Delete all messages from a channel.
        send_file: Send a file to a channel.
        export_text_messages: Export text messages from a channel.
    """

    token = ""
    first_name = ""
    username = ""
    json_updates = []
    chat_count = 0
    chat_list = []
    commands = []
    channels = {}

    def __init__(self, token):
        self.token = token

        self.get_me(token)
        self.get_chats()

    def get_me(self, token: str):
        """
        Get bot information.

        Args:
            token (str): Bot token.
        """

        url = f"https://api.telegram.org/bot{token}/getMe"
        response = requests.get(url)

        if not response.json()["ok"]:
            print("[!] Error: Couldn't get bot information. Token might be invalid.")
            sys.exit(1)

        first_name = response.json()["result"]["first_name"]
        username = response.json()["result"]["username"]

        self.first_name = first_name
        self.username = username

    def get_updates(self, token: str) -> dict:
        """
        Get updates from bot.

        Args:
            token (str): Bot token.

        Returns:
            response.json(): JSON response from getUpdates.
        """

        url = f"https://api.telegram.org/bot{token}/getUpdates"
        response = requests.get(url)

        return response.json()

    def update(self) -> bool:
        """
        Update bot's JSON updates.

        Returns:
            True: If updates are found.
            False: If no updates are found.
        """

        new_json_updates = self.get_updates(self.token)
        if new_json_updates == self.json_updates:
            return False
        else:
            self.json_updates = new_json_updates
            self.get_chats()
            return True

    def get_chats(self):
        """
        Get list of chats bot is part of.
        """
        try:
            if not self.json_updates["ok"]:
                print("[-] Error: Couldn't load chat history.")
                return [""]
        except TypeError:
            return [""]

        chats = []
        left_chats = []

        for chat in self.json_updates["result"]:
            if "message" not in chat:
                continue

            chats.append(
                f'{chat["message"]["chat"].get("title") or "Private Chat"}$$$$${chat["message"]["chat"]["id"]}'
            )

            if "left_chat_member" in chat["message"]:
                left_chats.append(
                    f'{chat["message"]["chat"].get("title") or "Private Chat"}$$$$${chat["message"]["chat"]["id"]}'
                )

        chats = sorted(set(chats), key=lambda x: x.split("$$$$$")[0])
        for chat in left_chats:
            if chat in chats:
                chats.remove(chat)

        for chat in chats:
            self.add_channel(chat.split("$$$$$")[1])

        self.chat_list = chats
        self.chat_count = len(chats)

    def add_channel(self, chat_id: str) -> bool:
        """
        Add channel to bot's channels.

        Args:
            chat_id (str): Chat ID.

        Returns:
            True: If channel is added successfully.
            False: If channel is already in bot's channels.
        """

        if self.is_in_channel(chat_id) and chat_id not in self.channels:
            self.channels[chat_id] = Channel(chat_id, self.token)
            return True
        else:
            return False

    def leave_channel(self, chat_id: str) -> bool:
        """
        Leave a channel.

        Args:
            chat_id (str): Chat ID.

        Returns:
            True: If channel is left successfully.
            False: If not.
        """

        url = f"https://api.telegram.org/bot{self.token}/leaveChat?chat_id={chat_id}"
        response = requests.get(url)

        if response.json()["ok"]:
            return True
        else:
            return False

    def is_in_channel(self, chat_id: str) -> bool:
        """
        Check if bot is part of a channel.

        Args:
            chat_id (str): Chat ID.

        Returns:
            True: If bot is part of the channel.
            False: If not.
        """

        url = f"https://api.telegram.org/bot{self.token}/getChat?chat_id={chat_id}"
        response = requests.get(url)
        if response.json()["result"]["type"] == "private":
            return True

        url = f"https://api.telegram.org/bot{self.token}/getChatAdministrators?chat_id={chat_id}"
        response = requests.get(url)

        if response.json()["ok"]:
            return True
        else:
            return False

    def get_file_count(self, chat_id: str) -> tuple[dict, dict]:
        """
        Get file count in a channel.

        Args:
            chat_id (str): Chat ID.

        Returns:
            file_count_dict (dict): Dictionary of file counts.
        """

        file_count_dict = {
            "gif": 0,
            "photo": 0,
            "video": 0,
            "sticker": 0,
            "document": 0,
            "voice-message": 0,
        }

        document_extensions = {}

        self.channels[chat_id].get_messages()
        messages = self.channels[chat_id].all_messages

        for chat in messages:

            if "photo" in chat["result"]:
                file_count_dict["photo"] += 1

            elif "animation" in chat["result"]:
                file_count_dict["gif"] += 1

            elif "document" in chat["result"]:
                file_count_dict["document"] += 1
                extension = chat["result"]["document"]["file_name"].split(".")[-1]

                if extension in document_extensions:
                    document_extensions[extension] += 1
                else:
                    document_extensions[extension] = 1

            elif "voice" in chat["result"]:
                file_count_dict["voice-message"] += 1

            elif "sticker" in chat["result"]:
                file_count_dict["sticker"] += 1

            elif "video" in chat["result"]:
                file_count_dict["video"] += 1

        return file_count_dict, document_extensions

    def download_file(self, file_information: tuple[str, str, str], chat_id: str):
        """
        Download a file from a channel.

        Args:
            file_information (tuple): File information.
            chat_id (str): Chat ID.
        """

        file_id = file_information[0]
        file_name = file_information[1]
        if file_information[2]:
            file_name = f"{file_name}.{file_information[2]}"

        url = f"https://api.telegram.org/bot{self.token}/getFile?file_id={file_id}"
        response = requests.get(url)

        if not response.json()["ok"]:
            print("[-] File exceeded maximum size.")
            return

        file_path = response.json()["result"]["file_path"]

        download_url = f"https://api.telegram.org/file/bot{self.token}/{file_path}"

        if not os.path.exists(chat_id.replace("-", "")):
            os.makedirs(chat_id.replace("-", ""))

        try:
            response = requests.get(download_url)
            response.raise_for_status()

            with open(f"./{chat_id.replace('-', '')}/{file_name}", "wb") as file:
                file.write(response.content)
            print(
                f"File downloaded successfully: ./{chat_id.replace('-', '')}/{file_name}"
            )
        except Exception as e:
            print(f"Error downloading file: {e}")

    def download_all_files(self, chat_id: str, file_type: str | None = None):
        """
        Download all files from a channel.

        Args:
            chat_id (str): Chat ID.
            file_type (str): File type.
        """

        file_informations = []

        self.channels[chat_id].get_messages()
        messages = self.channels[chat_id].all_messages

        for chat in messages:

            if file_type == "documents":

                if "photo" in chat["result"]:
                    continue

                elif "animation" in chat["result"]:
                    continue

                elif "voice" in chat["result"]:
                    continue

                elif "video" in chat["result"]:
                    continue

                elif "document" in chat["result"]:
                    document = chat["result"]["document"]
                    file_informations.append(
                        [document["file_id"], document["file_name"], None]
                    )

            elif file_type == "media":
                if "photo" in chat["result"]:
                    photo = chat["result"]["photo"][-1]
                    file_informations.append(
                        [photo["file_id"], photo["file_unique_id"], "png"]
                    )

                elif "animation" in chat["result"]:
                    continue

                elif "voice" in chat["result"]:
                    voice = chat["result"]["voice"]
                    file_informations.append(
                        [voice["file_id"], voice["file_unique_id"], "mp3"]
                    )

                elif "video" in chat["result"]:
                    video = chat["result"]["video"]
                    file_informations.append(
                        [video["file_id"], video["file_unique_id"], "mp4"]
                    )

            else:
                if "photo" in chat["result"]:
                    photo = chat["result"]["photo"][-1]
                    file_informations.append(
                        [photo["file_id"], photo["file_unique_id"], "png"]
                    )

                elif "animation" in chat["result"]:
                    continue

                elif "voice" in chat["result"]:
                    voice = chat["result"]["voice"]
                    file_informations.append(
                        [voice["file_id"], voice["file_unique_id"], "mp3"]
                    )

                elif "video" in chat["result"]:
                    video = chat["result"]["video"]
                    file_informations.append(
                        [video["file_id"], video["file_unique_id"], "mp4"]
                    )

                elif "document" in chat["result"]:
                    document = chat["result"]["document"]
                    file_informations.append(
                        [document["file_id"], document["file_name"], None]
                    )

        for file_information in file_informations:
            self.download_file(file_information, chat_id)

    def send_message(self, chat_id: str, message: str) -> bool:
        """
        Send a message to a channel.

        Args:
            chat_id (str): Chat ID.
            message (str): Message.

        Returns:
            True: If message is sent successfully.
            False: If not.
        """

        url = f"https://api.telegram.org/bot{self.token}/sendMessage?chat_id={chat_id}&text={message}"
        response = requests.get(url)

        return response.json()["ok"]

    def delete_message(self, chat_id: str, message_id: str) -> bool:
        """
        Delete a message from a channel.

        Args:
            chat_id (str): Chat ID.
            message_id (str): Message ID.

        Returns:
            True: If message is deleted successfully.
            False: If not.
        """

        url = f"https://api.telegram.org/bot{self.token}/deleteMessage?chat_id={chat_id}&message_id={message_id}"
        response = requests.get(url)

        return response.json()["ok"]

    def delete_all_messages(self, chat_id: str) -> int:
        """
        Delete all messages from a channel.

        Args:
            chat_id (str): Chat ID.

        Returns:
            message_count (int): Number of messages deleted.
        """

        self.channels[chat_id].get_messages()
        print("[+] Deleting all messages...")

        message_count = 0
        for message in self.channels[chat_id].all_messages_ids:
            if self.delete_message(chat_id, message):
                message_count += 1

        return message_count

    def send_file(self, chat_id: str, file_path: str) -> bool:
        """
        Send a file to a channel.

        Args:
            chat_id (str): Chat ID.
            file_path (str): File path.

        Returns:
            True: If file is sent successfully.
            False: If not.
        """

        url = f"https://api.telegram.org/bot{self.token}/sendDocument?chat_id={chat_id}"
        files = {"document": open(file_path, "rb")}
        response = requests.post(url, files=files)

        return response.json()["ok"]

    def export_text_messages(self, chat_id: str) -> bool:
        """
        Export text messages from a channel.

        Args:
            chat_id (str): Chat ID.

        Returns:
            True: If messages are exported successfully.
            False: If not.
        """

        self.channels[chat_id].get_messages()

        messages = self.channels[chat_id].parsed_messages

        if not messages:
            return False

        if not os.path.exists(chat_id.replace("-", "")):
            os.makedirs(chat_id.replace("-", ""))

        with open(f"./{chat_id.replace('-', '')}/messages.txt", "w") as file:
            for message in messages:
                file.write(message + "\n")

        return True



from setuptools import setup, find_packages

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setup(
    name="telemal",
    version="1.2.20",
    description="Get intelligence on malicious Telegram channels using bot Token",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/nicolasmf/Telemal",
    author="TA1GA",
    license="MIT",
    packages=find_packages(),
    entry_points={
        "console_scripts": ["telemal=telemal.telemal:main"],
    },
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
    ],
)
